
function get_parameters()
    params = (
        sim_length = 500,
        num_households = 5000,
        I_0_fraction = 0.002,
        β_y = 0.0001,
        β_m = 0.0001,
        β_o = 1.0,
        recovery_rate = 1/5,
        π_base_y = -5.0,
        π_base_m = -5.0,
        π_base_o = 0.0,
        η = 0.0,
        κ = 0.0,
        ω = 0.00005,
        ω_en = 0.00,
        γ = 0.0,
        ξ = 5.0,
        notification_parameter = 0.001,
        vaccinator_prob = 0.2,
        app_user_fraction = 0.0,
        notification_threshold = 2,
        immunizing = true,
        immunization_delay = 14,
        immunization_begin_day =60, 
        infection_introduction_day = 180,
    )
    return params
end

function get_u_0(nodes,I_0_fraction,vaccinator_prob)
    is_vaccinator = rand(Random.default_rng(Threads.threadid()),nodes) .< vaccinator_prob
    status = fill(Susceptible,nodes)
    return status,is_vaccinator
end

function app_users(demographics,app_usage_prob)
    ymo_usage = [
        0.979,
        0.921,
        0.604,  
    ]
    is_app_user = Vector{Bool}(undef,length(demographics))
    @inbounds for i in eachindex(demographics)
        demo = demographics[i]
        is_app_user[i] = rand(Random.default_rng(Threads.threadid())) < app_usage_prob*ymo_usage[Int(demo)]
    end
    return is_app_user
end


mutable struct ModelSolution{T,InfNet,SocNet,WSMixingDist,RestMixingDist}
    sim_length::Int
    nodes::Int
    params::T
    u_next_inf::Vector{AgentStatus}
    u_next_vac::Vector{Bool}
    u_inf::Vector{AgentStatus}
    u_vac::Vector{Bool}
    covid_alert_notifications::Array{Bool,2}
    time_of_last_alert::Vector{Int}
    inf_network::InfNet
    soc_network::SocNet
    index_vectors::Vector{Vector{Int}}
    demographics::Vector{AgentDemographic}
    app_user::Vector{Bool}
    app_user_list::Vector{Int}
    app_user_index::Vector{Int}
    status_totals::Vector{Int}
    daily_vaccinators::Int
    daily_cases_by_age::Vector{Int}
    daily_immunized_by_age::Vector{Int}
    ws_matrix_tuple::WSMixingDist
    rest_matrix_tuple::RestMixingDist
    immunization_countdown::Vector{Int}
    function ModelSolution(sim_length,params::T,num_households) where T
        demographics,base_network,index_vectors = generate_population(num_households)
        nodes = length(demographics)
        pop_sizes = length.(index_vectors)
        map_symmetrize(m_tuple) = map(md -> symmetrize_means(pop_sizes,md), m_tuple)

        ws_matrix_tuple = map_symmetrize(workschool_mixing)
        rest_matrix_tuple = map_symmetrize(rest_mixing)

        is_app_user = app_users(demographics,params.app_user_fraction)
        app_user_list = zeros(nodes)
        app_user_index = findall(==(true),is_app_user)
        app_user_list[is_app_user] .= collect(1:length(app_user_index))
        
        u_0_inf,u_0_vac = get_u_0(nodes,params.I_0_fraction,params.vaccinator_prob)

        infected_mixing_graph,soc_mixing_graph = time_dep_mixing_graphs(sim_length,base_network,demographics,index_vectors,ws_matrix_tuple,rest_matrix_tuple)
      
        covid_alert_notifications = zeros(Bool,14,length(app_user_index)) #two weeks worth of values
        time_of_last_alert = fill(-1,length(app_user_index)) #time of last alert is negative if no alert has been recieved

        status_totals = [count(==(AgentStatus(i)), u_0_inf) for i in 1:AgentStatus.size]
        immunization_countdown = fill(-1, nodes) #immunization countdown is negative if not counting down
        return new{T,typeof(infected_mixing_graph),typeof(soc_mixing_graph),typeof(ws_matrix_tuple),typeof(rest_matrix_tuple)}(
            sim_length,
            nodes,
            params,
            u_0_inf,
            u_0_vac,
            copy(u_0_inf),
            copy(u_0_vac),
            covid_alert_notifications,
            time_of_last_alert,
            infected_mixing_graph,
            soc_mixing_graph,
            index_vectors,
            demographics,
            is_app_user,
            app_user_list,
            app_user_index,
            status_totals,
            0,
            [0,0,0],
            [0,0,0],
            ws_matrix_tuple,
            rest_matrix_tuple,
            immunization_countdown
        )
    end
end
