
#remove Base.@propagate_inbounds during tests if you get segfaults or change anything,
# as that might mean there is an indexing bug that is not being caught,
#and therefore accessing the wrong memory is causing background errors

function contact_weight(β, contact_time) 
    return 1 - (1-β)^contact_time
end

function Φ(payoff,ξ)
    return 1 / (exp(-1*ξ*payoff))
end

Base.@propagate_inbounds @views function update_alert_durations!(t,modelsol) # Base.@propagate_inbounds 
    @unpack notification_parameter,notification_threshold = modelsol.params
    @unpack time_of_last_alert, app_user_index,inf_network,covid_alert_notifications,app_user = modelsol
    for (i,node) in enumerate(modelsol.app_user_index)
        for j in 2:14
            covid_alert_notifications[j-1,i] = covid_alert_notifications[j,i] #shift them all back  
        end
        total_weight_i = 0
        for mixing_graph in inf_network.graph_list[t]
            for j in neighbors(mixing_graph,node)
                if app_user[j]
                    total_weight_i+= get_weight(mixing_graph,GraphEdge(node,j))
                end
            end
        end
        coin_flip = 1 - (1 - notification_parameter)^total_weight_i
        r = rand(Random.default_rng(Threads.threadid()))
        if r < coin_flip
            covid_alert_notifications[end,i] = 1  #add the notifications for today
        else
            covid_alert_notifications[end,i] = 0
        end
        if sum(covid_alert_notifications[:,i])>=notification_threshold
            time_of_last_alert[i] = t
        end
    end
end

Base.@propagate_inbounds @views function update_infection_state!(t,modelsol)
    @unpack β_y,β_m,β_o,recovery_rate,immunizing,immunization_begin_day = modelsol.params
    @unpack u_inf,u_vac,u_next_inf,u_next_vac,demographics,inf_network,status_totals, immunization_countdown = modelsol
    
    modelsol.daily_cases_by_age .= 0
    modelsol.daily_immunized_by_age .= 0
    function agent_transition!(node, from::AgentStatus,to::AgentStatus) 
        immunization_countdown[node] = -1 
        status_totals[Int(from)] -= 1
        status_totals[Int(to)] += 1
        u_next_inf[node] = to
    end
    u_next_inf .= u_inf
    for i in 1:modelsol.nodes
        agent_status = u_inf[i]
        is_vaccinator = u_vac[i]
        agent_demo = demographics[i]
        if agent_status == Susceptible
            if is_vaccinator && immunizing && immunization_countdown[i] == -1 && t> immunization_begin_day
                immunization_countdown[i] = 14
            else
                for mixing_graph in inf_network.graph_list[t]
                    for j in neighbors(mixing_graph,i)    
                        if u_inf[j] == Infected && u_next_inf[i] != Infected
                            
                            β_vec = (β_y,β_m,β_o)
                            if rand(Random.default_rng(Threads.threadid())) < contact_weight(β_vec[Int(agent_demo)],get_weight(mixing_graph,GraphEdge(i,j)))
                                modelsol.daily_cases_by_age[Int(agent_demo)]+=1
                                agent_transition!(i, Susceptible,Infected)
                            end
                        end
                    end
                end
            end
        elseif agent_status == Infected
            if rand(Random.default_rng(Threads.threadid())) < recovery_rate
               agent_transition!(i, Infected,Recovered)
            end
        end
        
        if immunization_countdown[i] == 0 
            modelsol.daily_immunized_by_age[Int(agent_demo)] += 1
            agent_transition!(i, Susceptible,Immunized)
        elseif immunization_countdown[i]>0
            immunization_countdown[i] -= 1
        end
    end
end


Base.@propagate_inbounds @views function update_vaccination_opinion_state!(t,modelsol,total_infections)
    @unpack π_base_y,π_base_m,π_base_o, η,γ, κ, ω, ω_en,γ,ξ = modelsol.params
    @unpack demographics,time_of_last_alert, nodes, soc_network,u_vac,u_next_vac,app_user,app_user_list = modelsol
    app_user_pointer = 0
    for i in 1:nodes
        π_base = @SVector [π_base_y,π_base_m,π_base_o]
        vac_payoff = 0
        num_soc_nbrs = 0
        random_soc_network = sample(Random.default_rng(Threads.threadid()), soc_network.graph_list[t])

        if !isempty(neighbors(random_soc_network,i))
            random_neighbour = sample(Random.default_rng(Threads.threadid()), neighbors(random_soc_network.g,i))
            if u_vac[random_neighbour] == u_vac[i]
                vac_payoff += π_base[Int(demographics[i])] + total_infections*ω             
                if app_user[i] && time_of_last_alert[app_user_list[i]]>=0
                    vac_payoff += γ^(-1*(t - time_of_last_alert[app_user_list[i]])) * (η + total_infections*ω_en) 
                end
        
                if u_vac[i]
                    if rand(Random.default_rng(Threads.threadid())) < 1 - Φ(vac_payoff,ξ)
                        u_next_vac[i] = false
                    end
                else
                    if rand(Random.default_rng(Threads.threadid())) < Φ(vac_payoff,ξ)
                        u_next_vac[i] = true
                    end
                end
            end
        end
    end

    modelsol.daily_vaccinators = count(==(true),u_vac) #could maybe make this more efficient

end

function weighted_degree(node,network::TimeDepMixingGraph)
    weighted_degree = 0
    for g_list in network.graph_list
        for g in g_list
            for j in neighbors(g,node)
                weighted_degree += get_weight(g,GraphEdge(node,j))
            end
        end
    end
    return weighted_degree
end

function agents_step!(t,modelsol)

    remake!(modelsol.inf_network,modelsol.index_vectors)
    for network in modelsol.inf_network.graph_list[t] #this also resamples the soc network weights since they point to the same objects, but those are never used
        sample_mixing_graph!(network) #get new contact weights
    end

    if t == modelsol.params.infection_introduction_day
        init_indices = rand(Random.default_rng(Threads.threadid()), 1:modelsol.nodes, round(Int,modelsol.nodes*modelsol.params.I_0_fraction))
        modelsol.u_inf[init_indices] .= Infected
        modelsol.status_totals[Int(Infected)] += length(init_indices)
    end
    if t>modelsol.params.infection_introduction_day
        update_alert_durations!(t,modelsol)
    end
    update_vaccination_opinion_state!(t,modelsol,modelsol.status_totals[Int(Infected)])
    update_infection_state!(t,modelsol)

    modelsol.u_vac .= modelsol.u_next_vac
    modelsol.u_inf .= modelsol.u_next_inf
end



function solve!(modelsol,recordings...)

    for t in 1:modelsol.sim_length
        agents_step!(t,modelsol) 
        #advance agent states based on the new network
        for recording in recordings
            record!(t,modelsol,recording)
        end
    end
end


